@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -21,6 +20,6 @@
 #ifndef FLB_STRPTIME_H
 #define FLB_STRPTIME_H
 
-char *flb_strptime(const char *s, const char *format, struct tm *tm);
+char *flb_strptime(const char *s, const char *format, struct flb_tm *tm);
 
 #endif
