@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -33,7 +32,7 @@
 struct flb_hs_buf {
     int users;
     flb_sds_t data;
-    char *raw_data;
+    void *raw_data;
     size_t raw_size;
     struct mk_list _head;
 };
@@ -42,6 +41,7 @@ struct flb_hs {
     mk_ctx_t *ctx;             /* Monkey HTTP Context */
     int vid;                   /* Virtual Host ID     */
     int qid_metrics;           /* Metrics Message Queue ID    */
+    int qid_metrics_v2;        /* Metrics Message Queue ID for /api/v2 */
     int qid_storage;           /* Storage Message Queue ID    */
     int qid_health;            /* health Message Queue ID    */
 
@@ -55,7 +55,9 @@ struct flb_hs {
 
 struct flb_hs *flb_hs_create(const char *listen, const char *tcp_port,
                              struct flb_config *config);
+int flb_hs_push_health_metrics(struct flb_hs *hs, void *data, size_t size);
 int flb_hs_push_pipeline_metrics(struct flb_hs *hs, void *data, size_t size);
+int flb_hs_push_metrics(struct flb_hs *hs, void *data, size_t size);
 int flb_hs_push_storage_metrics(struct flb_hs *hs, void *data, size_t size);
 
 int flb_hs_destroy(struct flb_hs *ctx);
