@@ -1,17 +1,25 @@
-/*
- * Copyright (C) The c-ares project
+/* MIT License
  *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in
- * advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Copyright (c) The c-ares project and its contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  * SPDX-License-Identifier: MIT
  */
@@ -20,8 +28,6 @@
 #define ARES_TEST_H
 
 #include "ares_setup.h"
-#include "ares.h"
-
 #include "dns-proto.h"
 // Include ares internal file for DNS protocol constants
 #include "ares_nameser.h"
@@ -29,9 +35,6 @@
 #include "gtest/gtest.h"
 #include "gmock/gmock.h"
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
 #if defined(HAVE_USER_NAMESPACE) && defined(HAVE_UTS_NAMESPACE)
 #  define HAVE_CONTAINER
 #endif
@@ -42,8 +45,11 @@
 #include <memory>
 #include <set>
 #include <string>
+#include <mutex>
+#include <thread>
 #include <utility>
 #include <vector>
+#include <chrono>
 
 namespace ares {
 
@@ -61,9 +67,23 @@ extern const std::vector<std::pair<int, bool>> both_families_both_modes;
 extern const std::vector<std::pair<int, bool>> ipv4_family_both_modes;
 extern const std::vector<std::pair<int, bool>> ipv6_family_both_modes;
 
+extern const std::vector<std::tuple<ares_evsys_t, int, bool>>
+  all_evsys_ipv4_family_both_modes;
+extern const std::vector<std::tuple<ares_evsys_t, int, bool>>
+  all_evsys_ipv6_family_both_modes;
+extern const std::vector<std::tuple<ares_evsys_t, int, bool>>
+  all_evsys_both_families_both_modes;
+
+extern const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_ipv4_family;
+extern const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_ipv6_family;
+extern const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_both_families;
+
 // Which parameters to use in tests
-extern std::vector<int>                        families;
-extern std::vector<std::pair<int, bool>>       families_modes;
+extern std::vector<int>                                 families;
+extern std::vector<std::tuple<ares_evsys_t, int>>       evsys_families;
+extern std::vector<std::pair<int, bool>>                families_modes;
+extern std::vector<std::tuple<ares_evsys_t, int, bool>> evsys_families_modes;
+
 
 // Process all pending work on ares-owned file descriptors, plus
 // optionally the given set-of-FDs + work function.
@@ -73,6 +93,12 @@ void                    ProcessWork(ares_channel_t                          *cha
                                     unsigned int                             cancel_ms = 0);
 std::set<ares_socket_t> NoExtraFDs();
 
+const char             *af_tostr(int af);
+const char             *mode_tostr(bool mode);
+std::string
+  PrintFamilyMode(const testing::TestParamInfo<std::pair<int, bool>> &info);
+std::string PrintFamily(const testing::TestParamInfo<int> &info);
+
 // Test fixture that ensures library initialization, and allows
 // memory allocations to be failed.
 class LibraryTest : public ::testing::Test {
@@ -106,6 +132,7 @@ private:
   static bool                  ShouldAllocFail(size_t size);
   static unsigned long long    fails_;
   static std::map<size_t, int> size_fails_;
+  static std::mutex            lock_;
 };
 
 // Test fixture that uses a default channel.
@@ -191,6 +218,7 @@ protected:
   ares_channel_t *channel_;
 };
 
+
 // Mock DNS server to allow responses to be scripted by tests.
 class MockServer {
 public:
@@ -205,12 +233,23 @@ public:
   // with the value from the request.
   void SetReplyData(const std::vector<byte> &reply)
   {
-    reply_ = reply;
+    exact_reply_ = reply;
+    reply_ = nullptr;
   }
 
   void SetReply(const DNSPacket *reply)
   {
-    SetReplyData(reply->data());
+    reply_ = reply;
+    exact_reply_.clear();
+  }
+
+  // Set the reply to be sent next as well as the request (in string form) that
+  // the server should expect to receive; the query ID field in the reply will
+  // be overwritten with the value from the request.
+  void SetReplyExpRequest(const DNSPacket *reply, const std::string &request)
+  {
+    expected_request_ = request;
+    reply_ = reply;
   }
 
   void SetReplyQID(int qid)
@@ -220,6 +259,8 @@ public:
 
   void Disconnect()
   {
+    reply_ = nullptr;
+    exact_reply_.clear();
     for (ares_socket_t fd : connfds_) {
       sclose(fd);
     }
@@ -248,8 +289,8 @@ public:
 
 private:
   void           ProcessRequest(ares_socket_t fd, struct sockaddr_storage *addr,
-                                ares_socklen_t addrlen, int qid, const std::string &name,
-                                int rrtype);
+                                ares_socklen_t addrlen, const std::string &reqstr,
+                                int qid, const char *name, int rrtype);
   void           ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr,
                                ares_socklen_t addrlen, byte *data, int len);
   unsigned short udpport_;
@@ -257,7 +298,9 @@ private:
   ares_socket_t  udpfd_;
   ares_socket_t  tcpfd_;
   std::set<ares_socket_t> connfds_;
-  std::vector<byte>       reply_;
+  std::vector<byte>       exact_reply_;
+  const DNSPacket        *reply_;
+  std::string             expected_request_;
   int                     qid_;
   unsigned char          *tcp_data_;
   size_t                  tcp_data_len_;
@@ -317,6 +360,73 @@ public:
   }
 };
 
+class MockEventThreadOptsTest : public MockChannelOptsTest {
+public:
+  MockEventThreadOptsTest(int count, ares_evsys_t evsys, int family,
+                          bool force_tcp, struct ares_options *givenopts,
+                          int optmask)
+    : MockChannelOptsTest(count, family, force_tcp,
+                          FillOptionsET(&evopts_, givenopts, evsys),
+                          optmask | ARES_OPT_EVENT_THREAD)
+  {
+  }
+
+  ~MockEventThreadOptsTest()
+  {
+  }
+
+  static struct ares_options *FillOptionsET(struct ares_options *opts,
+                                            struct ares_options *givenopts,
+                                            ares_evsys_t         evsys)
+  {
+    if (givenopts) {
+      memcpy(opts, givenopts, sizeof(*opts));
+    } else {
+      memset(opts, 0, sizeof(*opts));
+    }
+    opts->evsys = evsys;
+    return opts;
+  }
+
+  void Process(unsigned int cancel_ms = 0);
+private:
+  struct ares_options evopts_;
+};
+
+class MockEventThreadTest
+  : public MockEventThreadOptsTest,
+    public ::testing::WithParamInterface<std::tuple<ares_evsys_t, int, bool>> {
+public:
+  MockEventThreadTest()
+    : MockEventThreadOptsTest(1, std::get<0>(GetParam()),
+                              std::get<1>(GetParam()), std::get<2>(GetParam()),
+                              nullptr, 0)
+  {
+  }
+};
+
+class MockUDPEventThreadTest
+  : public MockEventThreadOptsTest,
+    public ::testing::WithParamInterface<std::tuple<ares_evsys_t, int>> {
+public:
+  MockUDPEventThreadTest()
+    : MockEventThreadOptsTest(1, std::get<0>(GetParam()),
+                              std::get<1>(GetParam()), false, nullptr, 0)
+  {
+  }
+};
+
+class MockTCPEventThreadTest
+  : public MockEventThreadOptsTest,
+    public ::testing::WithParamInterface<std::tuple<ares_evsys_t, int>> {
+public:
+  MockTCPEventThreadTest()
+    : MockEventThreadOptsTest(1, std::get<0>(GetParam()),
+                              std::get<1>(GetParam()), true, nullptr, 0)
+  {
+  }
+};
+
 // gMock action to set the reply for a mock server.
 ACTION_P2(SetReplyData, mockserver, data)
 {
@@ -328,6 +438,13 @@ ACTION_P2(SetReply, mockserver, reply)
   mockserver->SetReply(reply);
 }
 
+// gMock action to set the reply for a mock server, as well as the request (in
+// string form) that the server should expect to receive.
+ACTION_P3(SetReplyExpRequest, mockserver, reply, request)
+{
+  mockserver->SetReplyExpRequest(reply, request);
+}
+
 ACTION_P2(SetReplyQID, mockserver, qid)
 {
   mockserver->SetReplyQID(qid);
@@ -340,7 +457,7 @@ ACTION_P2(CancelChannel, mockserver, channel)
 }
 
 // gMock action to disconnect all connections.
-ACTION_P2(Disconnect, mockserver)
+ACTION_P(Disconnect, mockserver)
 {
   mockserver->Disconnect();
 }
@@ -439,6 +556,8 @@ void          HostCallback(void *data, int status, int timeouts,
                            struct hostent *hostent);
 void SearchCallback(void *data, int status, int timeouts, unsigned char *abuf,
                     int alen);
+void SearchCallbackDnsRec(void *data, ares_status_t status, size_t timeouts,
+                          const ares_dns_record_t *dnsrec);
 void NameInfoCallback(void *data, int status, int timeouts, char *node,
                       char *service);
 void AddrInfoCallback(void *data, int status, int timeouts,
