@@ -2,7 +2,7 @@
 
 /*  CMetrics
  *  ========
- *  Copyright 2021 Eduardo Silva <eduardo@calyptia.com>
+ *  Copyright 2021-2022 The CMetrics Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -21,7 +21,6 @@
 #define CMT_MPACK_UTILS_H
 
 #include <cmetrics/cmetrics.h>
-#include <cmetrics/cmt_sds.h>
 #include <cmetrics/cmt_mpack_utils_defs.h>
 #include <mpack/mpack.h>
 
@@ -35,13 +34,13 @@ struct cmt_mpack_map_entry_callback_t {
 
 int cmt_mpack_consume_double_tag(mpack_reader_t *reader, double *output_buffer);
 int cmt_mpack_consume_uint_tag(mpack_reader_t *reader, uint64_t *output_buffer);
-int cmt_mpack_consume_string_tag(mpack_reader_t *reader, cmt_sds_t *output_buffer);
+int cmt_mpack_consume_string_tag(mpack_reader_t *reader, cfl_sds_t *output_buffer);
 int cmt_mpack_unpack_map(mpack_reader_t *reader, 
                          struct cmt_mpack_map_entry_callback_t *callback_list, 
                          void *context);
 int cmt_mpack_unpack_array(mpack_reader_t *reader, 
                            cmt_mpack_unpacker_entry_callback_fn_t entry_processor_callback, 
                            void *context);
-
+int cmt_mpack_peek_array_length(mpack_reader_t *reader);
 
 #endif
