@@ -27,6 +27,9 @@ struct mk_channel *mk_channel_new(int type, int fd)
     struct mk_channel *channel;
 
     channel = mk_mem_alloc(sizeof(struct mk_channel));
+    if (!channel) {
+        return NULL;
+    }
     channel->type   = type;
     channel->fd     = fd;
     channel->status = MK_CHANNEL_OK;
@@ -35,6 +38,12 @@ struct mk_channel *mk_channel_new(int type, int fd)
     return channel;
 }
 
+int mk_channel_release(struct mk_channel *channel)
+{
+    mk_mem_free(channel);
+    return 0;
+}
+
 static inline size_t channel_write_in_file(struct mk_channel *channel,
                                            struct mk_stream_input *in)
 {
