@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -23,12 +22,6 @@
 #ifndef FLB_FILTER_GEOIP2_H
 #define FLB_FILTER_GEOIP2_H
 
-struct geoip2_lookup_key {
-    char *key;
-    int key_len;
-    struct mk_list _head;
-};
-
 struct geoip2_record {
     char *lookup_key;
     char *key;
@@ -40,10 +33,12 @@ struct geoip2_record {
 };
 
 struct geoip2_ctx {
+    flb_sds_t database;
     MMDB_s *mmdb;
     int lookup_keys_num;
     int records_num;
-    struct mk_list lookup_keys;
+    struct mk_list *lookup_keys;
+    struct mk_list *record_keys;
     struct mk_list records;
     struct flb_filter_instance *ins;
 };
