@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -31,6 +30,8 @@
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_regex.h>
 #include <fluent-bit/flb_record_accessor.h>
+#include <fluent-bit/flb_log_event_decoder.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 #include <msgpack.h>
 
 #include "grep.h"
@@ -54,6 +55,7 @@ static void delete_rules(struct grep_ctx *ctx)
 
 static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
 {
+    int first_rule = GREP_NO_RULE;
     flb_sds_t tmp;
     struct mk_list *head;
     struct mk_list *split;
@@ -80,12 +82,22 @@ static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
             rule->type = GREP_EXCLUDE;
         }
         else {
-            flb_plg_error(ctx->ins, "unknown rule type '%s'", kv->key);
-            delete_rules(ctx);
+            /* Other property. Skip */
             flb_free(rule);
-            return -1;
+            continue;
         }
 
+        if (ctx->logical_op != GREP_LOGICAL_OP_LEGACY && first_rule != GREP_NO_RULE) {
+            /* 'AND'/'OR' case */
+            if (first_rule != rule->type) {
+                flb_plg_error(ctx->ins, "Both 'regex' and 'exclude' are set.");
+                delete_rules(ctx);
+                flb_free(rule);
+                return -1;
+            }
+        }
+        first_rule = rule->type;
+
         /* As a value we expect a pair of field name and a regular expression */
         split = flb_utils_split(kv->val, ' ', 1);
         if (mk_list_size(split) != 2) {
@@ -114,6 +126,13 @@ static int set_rules(struct grep_ctx *ctx, struct flb_filter_instance *f_ins)
         /* Get remaining content (regular expression) */
         sentry = mk_list_entry_last(split, struct flb_split_entry, _head);
         rule->regex_pattern = flb_strndup(sentry->value, sentry->len);
+        if (rule->regex_pattern == NULL) {
+            flb_errno();
+            delete_rules(ctx);
+            flb_free(rule);
+            flb_utils_split_free(split);
+            return -1;
+        }
 
         /* Release split */
         flb_utils_split_free(split);
@@ -179,6 +198,8 @@ static int cb_grep_init(struct flb_filter_instance *f_ins,
                         void *data)
 {
     int ret;
+    size_t len;
+    const char* val;
     struct grep_ctx *ctx;
 
     /* Create context */
@@ -187,9 +208,33 @@ static int cb_grep_init(struct flb_filter_instance *f_ins,
         flb_errno();
         return -1;
     }
+    if (flb_filter_config_map_set(f_ins, ctx) < 0) {
+        flb_errno();
+        flb_plg_error(f_ins, "configuration error");
+        flb_free(ctx);
+        return -1;
+    }
     mk_list_init(&ctx->rules);
     ctx->ins = f_ins;
 
+    ctx->logical_op = GREP_LOGICAL_OP_LEGACY;
+    val = flb_filter_get_property("logical_op", f_ins);
+    if (val != NULL) {
+        len = strlen(val);
+        if (len == 3 && strncasecmp("AND", val, len) == 0) {
+            flb_plg_info(ctx->ins, "AND mode");
+            ctx->logical_op = GREP_LOGICAL_OP_AND;
+        }
+        else if (len == 2 && strncasecmp("OR", val, len) == 0) {
+            flb_plg_info(ctx->ins, "OR mode");
+            ctx->logical_op = GREP_LOGICAL_OP_OR;
+        }
+        else if (len == 6 && strncasecmp("legacy", val, len) == 0) {
+            flb_plg_info(ctx->ins, "legacy mode");
+            ctx->logical_op = GREP_LOGICAL_OP_LEGACY;
+        }
+    }
+
     /* Load rules */
     ret = set_rules(ctx, f_ins);
     if (ret == -1) {
@@ -202,6 +247,42 @@ static int cb_grep_init(struct flb_filter_instance *f_ins,
     return 0;
 }
 
+static inline int grep_filter_data_and_or(msgpack_object map, struct grep_ctx *ctx)
+{
+    ssize_t ra_ret;
+    int found = FLB_FALSE;
+    struct mk_list *head;
+    struct grep_rule *rule;
+
+    /* For each rule, validate against map fields */
+    mk_list_foreach(head, &ctx->rules) {
+        found = FLB_FALSE;
+        rule = mk_list_entry(head, struct grep_rule, _head);
+
+        ra_ret = flb_ra_regex_match(rule->ra, map, rule->regex, NULL);
+        if (ra_ret > 0) {
+            found = FLB_TRUE;
+        }
+
+        if (ctx->logical_op == GREP_LOGICAL_OP_OR && found == FLB_TRUE) {
+            /* OR case: One rule is matched. */
+            goto grep_filter_data_and_or_end;
+        }
+        else if (ctx->logical_op == GREP_LOGICAL_OP_AND && found == FLB_FALSE) {
+            /* AND case: One rule is not matched */
+            goto grep_filter_data_and_or_end;
+        }
+    }
+
+ grep_filter_data_and_or_end:
+    if (rule->type == GREP_REGEX) {
+        return found ? GREP_RET_KEEP : GREP_RET_EXCLUDE;
+    }
+
+    /* rule is exclude */
+    return found ? GREP_RET_EXCLUDE : GREP_RET_KEEP;
+}
+
 static int cb_grep_filter(const void *data, size_t bytes,
                           const char *tag, int tag_len,
                           void **out_buf, size_t *out_size,
@@ -213,56 +294,101 @@ static int cb_grep_filter(const void *data, size_t bytes,
     int ret;
     int old_size = 0;
     int new_size = 0;
-    msgpack_unpacked result;
     msgpack_object map;
-    msgpack_object root;
-    size_t off = 0;
+    struct flb_log_event_encoder log_encoder;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
+    struct grep_ctx *ctx;
+
     (void) f_ins;
     (void) i_ins;
     (void) config;
-    msgpack_sbuffer tmp_sbuf;
-    msgpack_packer tmp_pck;
 
-    /* Create temporary msgpack buffer */
-    msgpack_sbuffer_init(&tmp_sbuf);
-    msgpack_packer_init(&tmp_pck, &tmp_sbuf, msgpack_sbuffer_write);
+    ctx = (struct grep_ctx *) context;
 
-    /* Iterate each item array and apply rules */
-    msgpack_unpacked_init(&result);
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        root = result.data;
-        if (root.type != MSGPACK_OBJECT_ARRAY) {
-            continue;
-        }
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
 
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
+
+        return FLB_FILTER_NOTOUCH;
+    }
+
+    ret = flb_log_event_encoder_init(&log_encoder,
+                                     FLB_LOG_EVENT_FORMAT_DEFAULT);
+
+    if (ret != FLB_EVENT_ENCODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event encoder initialization error : %d", ret);
+
+        flb_log_event_decoder_destroy(&log_decoder);
+
+        return FLB_FILTER_NOTOUCH;
+    }
+
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
         old_size++;
 
         /* get time and map */
-        map  = root.via.array.ptr[1];
+        map  = *log_event.body;
+
+        if (ctx->logical_op == GREP_LOGICAL_OP_LEGACY) {
+            ret = grep_filter_data(map, ctx);
+        }
+        else {
+            ret = grep_filter_data_and_or(map, ctx);
+        }
 
-        ret = grep_filter_data(map, context);
         if (ret == GREP_RET_KEEP) {
-            msgpack_pack_object(&tmp_pck, root);
+            ret = flb_log_event_encoder_emit_raw_record(
+                             &log_encoder,
+                             log_decoder.record_base,
+                             log_decoder.record_length);
+
             new_size++;
         }
         else if (ret == GREP_RET_EXCLUDE) {
             /* Do nothing */
         }
     }
-    msgpack_unpacked_destroy(&result);
+
+    if (ret == FLB_EVENT_DECODER_ERROR_INSUFFICIENT_DATA &&
+        log_decoder.offset == bytes) {
+        ret = FLB_EVENT_ENCODER_SUCCESS;
+    }
+
+    flb_log_event_decoder_destroy(&log_decoder);
 
     /* we keep everything ? */
     if (old_size == new_size) {
+        flb_log_event_encoder_destroy(&log_encoder);
+
         /* Destroy the buffer to avoid more overhead */
-        msgpack_sbuffer_destroy(&tmp_sbuf);
         return FLB_FILTER_NOTOUCH;
     }
 
-    /* link new buffers */
-    *out_buf   = tmp_sbuf.data;
-    *out_size = tmp_sbuf.size;
+    if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+        *out_buf  = log_encoder.output_buffer;
+        *out_size = log_encoder.output_length;
 
-    return FLB_FILTER_MODIFIED;
+        ret = FLB_FILTER_MODIFIED;
+
+        flb_log_event_encoder_claim_internal_buffer_ownership(&log_encoder);
+    }
+    else {
+        flb_plg_error(ctx->ins,
+                      "Log event encoder error : %d", ret);
+
+        ret = FLB_FILTER_NOTOUCH;
+    }
+
+    flb_log_event_decoder_destroy(&log_decoder);
+    flb_log_event_encoder_destroy(&log_encoder);
+
+    return ret;
 }
 
 static int cb_grep_exit(void *data, struct flb_config *config)
@@ -278,11 +404,31 @@ static int cb_grep_exit(void *data, struct flb_config *config)
     return 0;
 }
 
+static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_STR, "regex", NULL,
+     FLB_CONFIG_MAP_MULT, FLB_FALSE, 0,
+     "Keep records in which the content of KEY matches the regular expression."
+    },
+    {
+     FLB_CONFIG_MAP_STR, "exclude", NULL,
+     FLB_CONFIG_MAP_MULT, FLB_FALSE, 0,
+     "Exclude records in which the content of KEY matches the regular expression."
+    },
+    {
+     FLB_CONFIG_MAP_STR, "logical_op", "legacy",
+     0, FLB_FALSE, 0,
+     "Specify whether to use logical conjuciton or disjunction. legacy, AND and OR are allowed."
+    },
+    {0}
+};
+
 struct flb_filter_plugin filter_grep_plugin = {
     .name         = "grep",
     .description  = "grep events by specified field values",
     .cb_init      = cb_grep_init,
     .cb_filter    = cb_grep_filter,
     .cb_exit      = cb_grep_exit,
+    .config_map   = config_map,
     .flags        = 0
 };
