@@ -623,12 +623,43 @@ static int cb_log_to_metrics_init(struct flb_filter_instance *f_ins,
             return -1;
     }
 
-    input_ins = flb_input_new(config, "emitter", NULL, FLB_FALSE);
-    if (!input_ins) {
-        flb_plg_error(f_ins, "cannot create metrics emitter instance");
+    tmp = (char *) flb_filter_get_property("emitter_name", f_ins);
+    /* If emitter_name is not set, use the default name */
+    if (tmp == NULL) {
+        tmp = (char *) flb_filter_name(f_ins);
+        ctx->emitter_name = flb_sds_create_size(64);
+        ctx->emitter_name = flb_sds_printf(&ctx->emitter_name, "emitter_for_%s", tmp);
+    }
+    else {
+        ctx->emitter_name = flb_sds_create(tmp);
+    }
+
+    ret = flb_input_name_exists(ctx->emitter_name, config);
+    if (ret == FLB_TRUE) {
+        flb_plg_error(f_ins, "emitter_name '%s' already exists",
+                      ctx->emitter_name);
+        flb_sds_destroy(ctx->emitter_name);
         log_to_metrics_destroy(ctx);
         return -1;
     }
+    input_ins = flb_input_new(config, "emitter", NULL, FLB_FALSE);
+    if (!input_ins) {
+        flb_plg_error(f_ins, "cannot create metrics emitter instance");
+        flb_sds_destroy(ctx->emitter_name);
+        log_to_metrics_destroy(ctx);
+        return -1;
+    }
+    /* Set the alias for emitter */
+    ret = flb_input_set_property(input_ins, "alias", ctx->emitter_name);
+    if (ret == -1) {
+        flb_plg_warn(ctx->ins,
+                     "cannot set emitter_name");
+        flb_sds_destroy(ctx->emitter_name);
+        log_to_metrics_destroy(ctx);
+        return -1;
+    }
+
+    flb_sds_destroy(ctx->emitter_name);
 
     /* Set the storage type for emitter */
     ret = flb_input_set_property(input_ins, "storage.type", "memory");
@@ -638,6 +669,11 @@ static int cb_log_to_metrics_init(struct flb_filter_instance *f_ins,
         return -1;
     }
 
+    /* Set the emitter_mem_buf_limit */
+    if(ctx->emitter_mem_buf_limit > 0) {
+        input_ins->mem_buf_limit = ctx->emitter_mem_buf_limit;
+    }
+
     /* Initialize emitter plugin */
     ret = flb_input_instance_init(input_ins, config);
     if (ret == -1) {
@@ -952,6 +988,18 @@ static struct flb_config_map config_map[] = {
      offsetof(struct log_to_metrics_ctx, tag),
      "Metric Tag"
     },
+    {
+     FLB_CONFIG_MAP_STR, "emitter_name", NULL,
+     FLB_FALSE, FLB_TRUE,
+     offsetof(struct log_to_metrics_ctx, emitter_name), NULL
+    },
+    {
+     FLB_CONFIG_MAP_SIZE, "emitter_mem_buf_limit", FLB_MEM_BUF_LIMIT_DEFAULT,
+     FLB_FALSE, FLB_TRUE,
+     offsetof(struct log_to_metrics_ctx, emitter_mem_buf_limit),
+     "set a buffer limit to restrict memory usage of metrics emitter"
+    },
+
     {0}
 };
 
