@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -24,10 +23,11 @@
 #include <stdint.h>
 #include <unistd.h>
 #include <fluent-bit/flb_input.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 #include <msgpack.h>
 
-#define DEFAULT_INTERVAL_SEC  1
-#define DEFAULT_INTERVAL_NSEC 0
+#define DEFAULT_INTERVAL_SEC  "1"
+#define DEFAULT_INTERVAL_NSEC "0"
 
 #define FLB_CMD_LEN 256
 #define FLB_IN_PROC_NAME "in_proc"
@@ -57,9 +57,9 @@ struct flb_in_proc_config {
     uint8_t  alive;
 
     /* Checking process */
-    char*  proc_name;
-    pid_t  pid;
-    size_t len_proc_name;
+    flb_sds_t  proc_name;
+    pid_t      pid;
+    size_t     len_proc_name;
 
     /* Time interval check */
     int interval_sec;
@@ -72,6 +72,7 @@ struct flb_in_proc_config {
     uint8_t fds;
 
     struct flb_input_instance *ins;
+    struct flb_log_event_encoder *log_encoder;
 };
 
 #endif /*FLB_IN_PROC_H*/
