@@ -946,7 +946,13 @@ int flb_tail_file_append(char *path, struct stat *st, int mode,
     if (flb_tail_file_exists(st, ctx) == FLB_TRUE) {
         return -1;
     }
-
+   
+    #ifdef __linux__
+    if (ctx->file_cache_advise) {
+        flb_plg_debug(ctx->ins, "file will be read in POSIX_FADV_DONTNEED mode %s", path);
+    }
+    #endif
+    
     fd = open(path, O_RDONLY);
     if (fd == -1) {
         flb_errno();
@@ -1435,6 +1441,15 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
         file_buffer_capacity = (file->buf_size - file->buf_len) - 1;
     }
 
+    #ifdef __linux__
+    if (ctx->file_cache_advise) {
+        if (posix_fadvise(file->fd, 0, 0, POSIX_FADV_DONTNEED) == -1) {
+            flb_errno();
+            flb_plg_error(ctx->ins, "error during posix_fadvise");
+        }
+    }
+    #endif
+    
     read_size = file_buffer_capacity;
 
     if (file->decompression_context != NULL) {
