@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -20,14 +19,19 @@
 
 #include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_http_client.h>
+#include <fluent-bit/flb_base64.h>
+#include <fluent-bit/flb_crypto.h>
+#include <fluent-bit/flb_hmac.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_utils.h>
 #include <fluent-bit/flb_time.h>
+#include <fluent-bit/flb_log_event_decoder.h>
+#include <fluent-bit/flb_record_accessor.h>
+#include <fluent-bit/flb_ra_key.h>
 #include <msgpack.h>
 
 #include "azure.h"
 #include "azure_conf.h"
-#include <mbedtls/base64.h>
 
 static int cb_azure_init(struct flb_output_instance *ins,
                           struct flb_config *config, void *data)
@@ -45,18 +49,14 @@ static int cb_azure_init(struct flb_output_instance *ins,
 }
 
 static int azure_format(const void *in_buf, size_t in_bytes,
+                        flb_sds_t tag, flb_sds_t *tag_val_out,
                         char **out_buf, size_t *out_size,
                         struct flb_azure *ctx)
 {
     int i;
     int array_size = 0;
     int map_size;
-    size_t off = 0;
     double t;
-    struct flb_time tm;
-    msgpack_unpacked result;
-    msgpack_object root;
-    msgpack_object *obj;
     msgpack_object map;
     msgpack_object k;
     msgpack_object v;
@@ -65,39 +65,88 @@ static int azure_format(const void *in_buf, size_t in_bytes,
     msgpack_sbuffer tmp_sbuf;
     msgpack_packer tmp_pck;
     flb_sds_t record;
+    char time_formatted[32];
+    size_t s;
+    struct tm tms;
+    int len;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
+    flb_sds_t tmp = NULL;
+    int ret;
 
     /* Count number of items */
     array_size = flb_mp_count(in_buf, in_bytes);
-    msgpack_unpacked_init(&result);
+
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) in_buf, in_bytes);
+
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(ctx->ins,
+                      "Log event decoder initialization error : %d", ret);
+
+        return -1;
+    }
 
     /* Create temporary msgpack buffer */
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
     msgpack_pack_array(&mp_pck, array_size);
 
-    off = 0;
-    while (msgpack_unpack_next(&result, in_buf, in_bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        root = result.data;
-
-        /* Get timestamp */
-        flb_time_pop_from_msgpack(&tm, &result, &obj);
-        t = flb_time_to_double(&tm);
-
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
         /* Create temporary msgpack buffer */
         msgpack_sbuffer_init(&tmp_sbuf);
         msgpack_packer_init(&tmp_pck, &tmp_sbuf, msgpack_sbuffer_write);
 
-        map = root.via.array.ptr[1];
+        map = *log_event.body;
         map_size = map.via.map.size;
 
+        if (ctx->log_type_key) {
+            tmp = flb_ra_translate(ctx->ra_prefix_key,
+                                tag, flb_sds_len(tag),
+                                map, NULL);
+            if (!tmp) {
+                flb_plg_error(ctx->ins, "Tagged record translation failed!");
+            }
+            else if (flb_sds_is_empty(tmp)) {
+                flb_plg_warn(ctx->ins, "Record accessor key not matched");
+                flb_sds_destroy(tmp);
+            }
+            else {
+                /* tag_val_out must be destroyed by the caller */
+                *tag_val_out = tmp;
+            }
+        }
+
         msgpack_pack_map(&mp_pck, map_size + 1);
 
         /* Append the time key */
         msgpack_pack_str(&mp_pck, flb_sds_len(ctx->time_key));
         msgpack_pack_str_body(&mp_pck,
-                              ctx->time_key,
-                              flb_sds_len(ctx->time_key));
-        msgpack_pack_double(&mp_pck, t);
+                            ctx->time_key,
+                            flb_sds_len(ctx->time_key));
+
+        if (ctx->time_generated == FLB_TRUE) {
+            /* Append the time value as ISO 8601 */
+            gmtime_r(&log_event.timestamp.tm.tv_sec, &tms);
+
+            s = strftime(time_formatted, sizeof(time_formatted) - 1,
+                            FLB_PACK_JSON_DATE_ISO8601_FMT, &tms);
+
+            len = snprintf(time_formatted + s,
+                            sizeof(time_formatted) - 1 - s,
+                            ".%03" PRIu64 "Z",
+                            (uint64_t) log_event.timestamp.tm.tv_nsec / 1000000);
+
+            s += len;
+            msgpack_pack_str(&mp_pck, s);
+            msgpack_pack_str_body(&mp_pck, time_formatted, s);
+        } else {
+            /* Append the time value as millis.nanos */
+            t = flb_time_to_double(&log_event.timestamp);
+
+            msgpack_pack_double(&mp_pck, t);
+        }
 
         /* Append original map k/v */
         for (i = 0; i < map_size; i++) {
@@ -114,13 +163,16 @@ static int azure_format(const void *in_buf, size_t in_bytes,
     record = flb_msgpack_raw_to_json_sds(mp_sbuf.data, mp_sbuf.size);
     if (!record) {
         flb_errno();
+
+        flb_log_event_decoder_destroy(&log_decoder);
         msgpack_sbuffer_destroy(&mp_sbuf);
-        msgpack_unpacked_destroy(&result);
+
         return -1;
     }
 
+    flb_log_event_decoder_destroy(&log_decoder);
+
     msgpack_sbuffer_destroy(&mp_sbuf);
-    msgpack_unpacked_destroy(&result);
 
     *out_buf = record;
     *out_size = flb_sds_len(record);
@@ -129,6 +181,7 @@ static int azure_format(const void *in_buf, size_t in_bytes,
 }
 
 static int build_headers(struct flb_http_client *c,
+                         flb_sds_t log_type,
                          size_t content_length,
                          struct flb_azure *ctx)
 {
@@ -142,7 +195,7 @@ static int build_headers(struct flb_http_client *c,
     flb_sds_t str_hash;
     struct tm tm = {0};
     unsigned char hmac_hash[32] = {0};
-    mbedtls_md_context_t mctx;
+    int result;
 
     /* Format Date */
     rfc1123date = flb_sds_create_size(32);
@@ -185,27 +238,36 @@ static int build_headers(struct flb_http_client *c,
     flb_sds_cat(str_hash, FLB_AZURE_RESOURCE, sizeof(FLB_AZURE_RESOURCE) - 1);
 
     /* Authorization signature */
-    mbedtls_md_init(&mctx);
-    mbedtls_md_setup(&mctx, mbedtls_md_info_from_type(MBEDTLS_MD_SHA256) , 1);
-    mbedtls_md_hmac_starts(&mctx, (unsigned char *) ctx->dec_shared_key,
-                           flb_sds_len(ctx->dec_shared_key));
-    mbedtls_md_hmac_update(&mctx, (unsigned char *) str_hash,
-                           flb_sds_len(str_hash));
-    mbedtls_md_hmac_finish(&mctx, hmac_hash);
-    mbedtls_md_free(&mctx);
+    result = flb_hmac_simple(FLB_HASH_SHA256,
+                             (unsigned char *) ctx->dec_shared_key,
+                             flb_sds_len(ctx->dec_shared_key),
+                             (unsigned char *) str_hash,
+                             flb_sds_len(str_hash),
+                             hmac_hash,
+                             sizeof(hmac_hash));
+
+    if (result != FLB_CRYPTO_SUCCESS) {
+        flb_sds_destroy(rfc1123date);
+        flb_sds_destroy(str_hash);
+        return -1;
+    }
 
     /* Encoded hash */
-    mbedtls_base64_encode((unsigned char *) &tmp, sizeof(tmp) - 1, &olen,
-                          hmac_hash, sizeof(hmac_hash));
+    result = flb_base64_encode((unsigned char *) &tmp, sizeof(tmp) - 1, &olen,
+                               hmac_hash, sizeof(hmac_hash));
     tmp[olen] = '\0';
 
     /* Append headers */
     flb_http_add_header(c, "User-Agent", 10, "Fluent-Bit", 10);
     flb_http_add_header(c, "Log-Type", 8,
-                        ctx->log_type, flb_sds_len(ctx->log_type));
+                        log_type, flb_sds_len(log_type));
     flb_http_add_header(c, "Content-Type", 12, "application/json", 16);
     flb_http_add_header(c, "x-ms-date", 9, rfc1123date,
                         flb_sds_len(rfc1123date));
+    if (ctx->time_generated == FLB_TRUE) {
+        /* Use time value as time-generated within azure */
+        flb_http_add_header(c, "time-generated-field", 20, ctx->time_key, flb_sds_len(ctx->time_key));
+    }
 
     size = 32 + flb_sds_len(ctx->customer_id) + olen;
     auth = flb_malloc(size);
@@ -229,8 +291,8 @@ static int build_headers(struct flb_http_client *c,
     return 0;
 }
 
-static void cb_azure_flush(const void *data, size_t bytes,
-                           const char *tag, int tag_len,
+static void cb_azure_flush(struct flb_event_chunk *event_chunk,
+                           struct flb_output_flush *out_flush,
                            struct flb_input_instance *i_ins,
                            void *out_context,
                            struct flb_config *config)
@@ -240,9 +302,10 @@ static void cb_azure_flush(const void *data, size_t bytes,
     char *buf_data;
     size_t buf_size;
     struct flb_azure *ctx = out_context;
-    struct flb_upstream_conn *u_conn;
+    struct flb_connection *u_conn;
     struct flb_http_client *c;
     flb_sds_t payload;
+    flb_sds_t final_log_type = NULL;
     (void) i_ins;
     (void) config;
 
@@ -253,7 +316,13 @@ static void cb_azure_flush(const void *data, size_t bytes,
     }
 
     /* Convert binary logs into a JSON payload */
-    ret = azure_format(data, bytes, &buf_data, &buf_size, ctx);
+    ret = azure_format(event_chunk->data, event_chunk->size,
+                      event_chunk->tag, &final_log_type, &buf_data, &buf_size, ctx);
+    /* If cannot get matching record using log_type_prefix, use log_type directly */
+    if (!final_log_type) {
+        final_log_type = ctx->log_type;
+    }
+
     if (ret == -1) {
         flb_upstream_conn_release(u_conn);
         FLB_OUTPUT_RETURN(FLB_ERROR);
@@ -266,7 +335,7 @@ static void cb_azure_flush(const void *data, size_t bytes,
     flb_http_buffer_size(c, FLB_HTTP_DATA_SIZE_MAX);
 
     /* Append headers and Azure signature */
-    ret = build_headers(c, flb_sds_len(payload), ctx);
+    ret = build_headers(c, final_log_type, flb_sds_len(payload), ctx);
     if (ret == -1) {
         flb_plg_error(ctx->ins, "error composing signature");
         flb_sds_destroy(payload);
@@ -298,6 +367,9 @@ static void cb_azure_flush(const void *data, size_t bytes,
     }
 
     /* Cleanup */
+    if (final_log_type != ctx->log_type) {
+        flb_sds_destroy(final_log_type);
+    }
     flb_http_client_destroy(c);
     flb_sds_destroy(payload);
     flb_upstream_conn_release(u_conn);
@@ -319,6 +391,52 @@ static int cb_azure_exit(void *data, struct flb_config *config)
     return 0;
 }
 
+/* Configuration properties map */
+static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_STR, "customer_id", NULL,
+     0, FLB_TRUE, offsetof(struct flb_azure, customer_id),
+     "Customer ID or WorkspaceID string."
+    },
+
+    {
+     FLB_CONFIG_MAP_STR, "shared_key", NULL,
+     0, FLB_TRUE, offsetof(struct flb_azure, shared_key),
+     "The primary or the secondary Connected Sources client authentication key."
+    },
+
+    {
+     FLB_CONFIG_MAP_STR, "log_type", FLB_AZURE_LOG_TYPE,
+     0, FLB_TRUE, offsetof(struct flb_azure, log_type),
+    "The name of the event type."
+    },
+
+    {
+     FLB_CONFIG_MAP_STR, "log_type_key", NULL,
+     0, FLB_TRUE, offsetof(struct flb_azure, log_type_key),
+     "If included, the value for this key will be looked upon in the record "
+     "and if present, will over-write the `log_type`. If the key/value "
+     "is not found in the record then the `log_type` option will be used. "
+    },
+
+    {
+     FLB_CONFIG_MAP_STR, "time_key", FLB_AZURE_TIME_KEY,
+     0, FLB_TRUE, offsetof(struct flb_azure, time_key),
+    "Optional parameter to specify the key name where the timestamp will be stored."
+    },
+
+    {
+     FLB_CONFIG_MAP_BOOL, "time_generated", "false",
+     0, FLB_TRUE, offsetof(struct flb_azure, time_generated),
+     "If enabled, the HTTP request header 'time-generated-field' will be included "
+     "so Azure can override the timestamp with the key specified by 'time_key' "
+     "option."
+    },
+
+    /* EOF */
+    {0}
+};
+
 struct flb_output_plugin out_azure_plugin = {
     .name         = "azure",
     .description  = "Send events to Azure HTTP Event Collector",
@@ -326,6 +444,9 @@ struct flb_output_plugin out_azure_plugin = {
     .cb_flush     = cb_azure_flush,
     .cb_exit      = cb_azure_exit,
 
+    /* Configuration */
+    .config_map     = config_map,
+
     /* Plugin flags */
     .flags          = FLB_OUTPUT_NET | FLB_IO_TLS,
 };
