@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -98,7 +97,7 @@ static int cb_prom_init(struct flb_output_instance *ins,
     }
 
     /* Hash table for metrics */
-    ctx->ht_metrics = flb_hash_create(FLB_HASH_EVICT_NONE, 32, 0);
+    ctx->ht_metrics = flb_hash_table_create(FLB_HASH_TABLE_EVICT_NONE, 32, 0);
     if (!ctx->ht_metrics) {
         flb_plg_error(ctx->ins, "could not initialize hash table for metrics");
         return -1;
@@ -127,7 +126,7 @@ static void append_labels(struct prom_exporter *ctx, struct cmt *cmt)
 }
 
 static int hash_store(struct prom_exporter *ctx, struct flb_input_instance *ins,
-                      cmt_sds_t buf)
+                      cfl_sds_t buf)
 {
     int ret;
     int len;
@@ -135,8 +134,8 @@ static int hash_store(struct prom_exporter *ctx, struct flb_input_instance *ins,
     len = strlen(ins->name);
 
     /* store/override the content into the hash table */
-    ret = flb_hash_add(ctx->ht_metrics, ins->name, len,
-                       buf, cmt_sds_len(buf));
+    ret = flb_hash_table_add(ctx->ht_metrics, ins->name, len,
+                             buf, cfl_sds_len(buf));
     if (ret < 0) {
         return -1;
     }
@@ -150,7 +149,7 @@ static flb_sds_t hash_format_metrics(struct prom_exporter *ctx)
     flb_sds_t buf;
 
     struct mk_list *head;
-    struct flb_hash_entry *entry;
+    struct flb_hash_table_entry *entry;
 
 
     buf = flb_sds_create_size(size);
@@ -160,22 +159,23 @@ static flb_sds_t hash_format_metrics(struct prom_exporter *ctx)
 
     /* Take every hash entry and compose one buffer with the whole content */
     mk_list_foreach(head, &ctx->ht_metrics->entries) {
-        entry = mk_list_entry(head, struct flb_hash_entry, _head_parent);
+        entry = mk_list_entry(head, struct flb_hash_table_entry, _head_parent);
         flb_sds_cat_safe(&buf, entry->val, entry->val_size);
     }
 
     return buf;
 }
 
-static void cb_prom_flush(const void *data, size_t bytes,
-                          const char *tag, int tag_len,
+static void cb_prom_flush(struct flb_event_chunk *event_chunk,
+                          struct flb_output_flush *out_flush,
                           struct flb_input_instance *ins, void *out_context,
                           struct flb_config *config)
 {
     int ret;
+    int add_ts;
     size_t off = 0;
     flb_sds_t metrics;
-    cmt_sds_t text;
+    cfl_sds_t text;
     struct cmt *cmt;
     struct prom_exporter *ctx = out_context;
 
@@ -184,7 +184,9 @@ static void cb_prom_flush(const void *data, size_t bytes,
      * convert to Prometheus text format and store the output in the
      * hash table for metrics.
      */
-    ret = cmt_decode_msgpack_create(&cmt, (char *) data, bytes, &off);
+    ret = cmt_decode_msgpack_create(&cmt,
+                                    (char *) event_chunk->data,
+                                    event_chunk->size, &off);
     if (ret != 0) {
         FLB_OUTPUT_RETURN(FLB_ERROR);
     }
@@ -192,15 +194,23 @@ static void cb_prom_flush(const void *data, size_t bytes,
     /* append labels set by config */
     append_labels(ctx, cmt);
 
+    /* add timestamp in the output format ? */
+    if (ctx->add_timestamp) {
+        add_ts = CMT_TRUE;
+    }
+    else {
+        add_ts = CMT_FALSE;
+    }
+
     /* convert to text representation */
-    text = cmt_encode_prometheus_create(cmt, CMT_TRUE);
+    text = cmt_encode_prometheus_create(cmt, add_ts);
     if (!text) {
         cmt_destroy(cmt);
         FLB_OUTPUT_RETURN(FLB_ERROR);
     }
     cmt_destroy(cmt);
 
-    if (cmt_sds_len(text) == 0) {
+    if (cfl_sds_len(text) == 0) {
         flb_plg_debug(ctx->ins, "context without metrics (empty)");
         cmt_encode_text_destroy(text);
         FLB_OUTPUT_RETURN(FLB_OK);
@@ -246,7 +256,7 @@ static int cb_prom_exit(void *data, struct flb_config *config)
     }
 
     if (ctx->ht_metrics) {
-        flb_hash_destroy(ctx->ht_metrics);
+        flb_hash_table_destroy(ctx->ht_metrics);
     }
 
     flb_kv_release(&ctx->kv_labels);
@@ -259,6 +269,12 @@ static int cb_prom_exit(void *data, struct flb_config *config)
 
 /* Configuration properties map */
 static struct flb_config_map config_map[] = {
+    {
+     FLB_CONFIG_MAP_BOOL, "add_timestamp", "false",
+     0, FLB_TRUE, offsetof(struct prom_exporter, add_timestamp),
+     "Add timestamp to every metric honoring collection time."
+    },
+
     {
      FLB_CONFIG_MAP_SLIST_1, "add_label", NULL,
      FLB_CONFIG_MAP_MULT, FLB_TRUE, offsetof(struct prom_exporter, add_labels),
