@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -175,32 +174,24 @@ int flb_router_io_set(struct flb_config *config)
     struct flb_output_instance *o_ins;
 
     /* Quick setup for 1:1 */
-    mk_list_foreach(i_head, &config->inputs) {
-        in_count++;
-    }
-    mk_list_foreach(o_head, &config->outputs) {
-        out_count++;
-    }
+    in_count = mk_list_size(&config->inputs);
+    out_count = mk_list_size(&config->outputs);
 
-    /* Just 1 input and 1 output */
+    /* Mostly used for command line tests */
     if (in_count == 1 && out_count == 1) {
-        i_ins = mk_list_entry_first(&config->inputs,
-                                    struct flb_input_instance, _head);
-        o_ins = mk_list_entry_first(&config->outputs,
-                                    struct flb_output_instance, _head);
+        i_ins = mk_list_entry_first(&config->inputs, struct flb_input_instance, _head);
+        o_ins = mk_list_entry_first(&config->outputs, struct flb_output_instance, _head);
 
-        if (flb_router_match_type(i_ins->event_type, o_ins) &&
-            !o_ins->match
+        if (!o_ins->match
 #ifdef FLB_HAVE_REGEX
             && !o_ins->match_regex
 #endif
             ) {
-            flb_debug("[router] default match rule %s:%s",
-                      i_ins->name, o_ins->name);
+
             o_ins->match = flb_sds_create_len("*", 1);
-            flb_router_connect(i_ins, o_ins);
-            return 0;
         }
+        flb_router_connect(i_ins, o_ins);
+        return 0;
     }
 
     /* N:M case, iterate all input instances */
@@ -240,25 +231,6 @@ int flb_router_io_set(struct flb_config *config)
 #endif
             )) {
 
-                if (!flb_router_match_type(i_ins->event_type, o_ins)) {
-                    if (i_ins->event_type == FLB_INPUT_LOGS) {
-                        flb_debug("[router] data generated by %s input are logs, "
-                                  "but matching destination plugin %s don't handle "
-                                  "logs. Skipping destination.",
-                                  flb_input_name(i_ins),
-                                  flb_output_name(o_ins));
-                    }
-                    else if (i_ins->event_type == FLB_INPUT_METRICS) {
-                        flb_debug("[router] data generated by %s input are metrics, "
-                                  "but matching destination plugin %s don't handle "
-                                  "metrics.Skipping destination.",
-                                  flb_input_name(i_ins),
-                                  flb_output_name(o_ins));
-                    }
-                    continue;
-                }
-
-
                 flb_debug("[router] match rule %s:%s",
                           i_ins->name, o_ins->name);
                 flb_router_connect(i_ins, o_ins);
