@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -29,10 +28,11 @@
 #include <fluent-bit/flb_kv.h>
 #include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_utils.h>
+#include <fluent-bit/flb_hmac.h>
+#include <fluent-bit/flb_hash.h>
 #include <fluent-bit/flb_http_client.h>
 #include <fluent-bit/flb_signv4.h>
 #include <fluent-bit/flb_aws_credentials.h>
-#include <mbedtls/sha256.h>
 
 #include <stdlib.h>
 #include <ctype.h>
@@ -66,21 +66,16 @@ static int hmac_sha256_sign(unsigned char out[32],
                             unsigned char *key, size_t key_len,
                             unsigned char *msg, size_t msg_len)
 {
-    mbedtls_md_context_t ctx;
-    mbedtls_md_type_t md_type = MBEDTLS_MD_SHA256;
+    int result;
 
-    mbedtls_md_init(&ctx);
-    mbedtls_md_setup(&ctx, mbedtls_md_info_from_type(md_type), 1);
+    result = flb_hmac_simple(FLB_HASH_SHA256,
+                             key, key_len,
+                             msg, msg_len,
+                             out, 32);
 
-    /* Start with the key */
-    mbedtls_md_hmac_starts(&ctx, key, key_len);
-
-    /* Update message */
-    mbedtls_md_hmac_update(&ctx, msg, msg_len);
-
-    /* Write digest to output buffer */
-    mbedtls_md_hmac_finish(&ctx, out);
-    mbedtls_md_free(&ctx);
+    if (result != FLB_CRYPTO_SUCCESS) {
+        return -1;
+    }
 
     return 0;
 }
@@ -93,7 +88,22 @@ static int kv_key_cmp(const void *a_arg, const void *b_arg)
 
     ret = strcmp(kv_a->key, kv_b->key);
     if (ret == 0) {
-        ret = strcmp(kv_a->val, kv_b->val);
+        /*
+         * NULL pointer is allowed in kv_a->val and kv_b->val.
+         * Handle NULL pointer cases before passing to strcmp.
+         */
+        if (kv_a->val == NULL && kv_b->val == NULL) {
+            ret = 0;
+        }
+        else if (kv_a->val == NULL) {
+            ret = -1;
+        }
+        else if (kv_b->val == NULL) {
+            ret = 1;
+        }
+        else {
+            ret = strcmp(kv_a->val, kv_b->val);
+        }
     }
 
     return ret;
@@ -321,6 +331,8 @@ static flb_sds_t url_params_format(char *params)
     ret = flb_slist_split_string(&split, params, '&', -1);
     if (ret == -1) {
         flb_error("[signv4] error processing given query string");
+        flb_slist_destroy(&split);
+        flb_kv_release(&list);
         return NULL;
     }
 
@@ -387,7 +399,7 @@ static flb_sds_t url_params_format(char *params)
         return flb_sds_create("");
     }
 
-    arr = flb_malloc(sizeof(struct flb_kv *) * items);
+    arr = flb_calloc(1, sizeof(struct flb_kv *) * items);
     if (!arr) {
         flb_errno();
         flb_kv_release(&list);
@@ -487,6 +499,9 @@ void headers_sanitize(struct mk_list *in_list, struct mk_list *out_list)
         kv = flb_kv_item_create_len(&out_tmp,
                                     kv->key, flb_sds_len(kv->key),
                                     v_start, v_end - v_start);
+        if (kv == NULL) {
+            continue;
+        }
         for (x = 0; x < flb_sds_len(kv->key); x++) {
             kv->key[x] = tolower(kv->key[x]);
         }
@@ -559,13 +574,18 @@ static flb_sds_t flb_signv4_canonical_request(struct flb_http_client *c,
                                               char *amzdate,
                                               char *security_token,
                                               int s3_mode,
+                                              struct mk_list *excluded_headers,
                                               flb_sds_t *signed_headers)
 {
     int i;
     int len;
     int items;
+    int all_items;
+    int excluded_items;
     int post_params = FLB_FALSE;
+    int result;
     size_t size;
+    int skip_header;
     char *val;
     struct flb_kv **arr;
     flb_sds_t cr;
@@ -576,8 +596,9 @@ static flb_sds_t flb_signv4_canonical_request(struct flb_http_client *c,
     struct flb_kv *kv;
     struct mk_list list_tmp;
     struct mk_list *head;
+    struct mk_list *head_2;
+    struct flb_slist_entry *sle;
     unsigned char sha256_buf[64] = {0};
-    mbedtls_sha256_context sha256_ctx;
 
     /* Size hint */
     size = strlen(c->uri) + (mk_list_size(&c->headers) * 64) + 256;
@@ -732,13 +753,22 @@ static flb_sds_t flb_signv4_canonical_request(struct flb_http_client *c,
     if (s3_mode == S3_MODE_UNSIGNED_PAYLOAD) {
         payload_hash = flb_sds_create("UNSIGNED-PAYLOAD");
     } else {
-        mbedtls_sha256_init(&sha256_ctx);
-        mbedtls_sha256_starts(&sha256_ctx, 0);
         if (c->body_len > 0 && post_params == FLB_FALSE) {
-            mbedtls_sha256_update(&sha256_ctx, (const unsigned char *) c->body_buf,
-                                  c->body_len);
+            result = flb_hash_simple(FLB_HASH_SHA256,
+                                     (unsigned char *) c->body_buf, c->body_len,
+                                     sha256_buf, sizeof(sha256_buf));
+        }
+        else {
+            result = flb_hash_simple(FLB_HASH_SHA256,
+                                     (unsigned char *) NULL, 0,
+                                     sha256_buf, sizeof(sha256_buf));
+        }
+
+        if (result != FLB_CRYPTO_SUCCESS) {
+            flb_error("[signv4] error hashing payload");
+            flb_sds_destroy(cr);
+            return NULL;
         }
-        mbedtls_sha256_finish(&sha256_ctx, sha256_buf);
 
         payload_hash = flb_sds_create_size(64);
         if (!payload_hash) {
@@ -792,9 +822,10 @@ static flb_sds_t flb_signv4_canonical_request(struct flb_http_client *c,
      * For every header registered, append it to the temporal array so we can sort them
      * later.
      */
-    items = mk_list_size(&list_tmp);
-    size = (sizeof(struct flb_kv *) * items);
-    arr = flb_malloc(size);
+    all_items = mk_list_size(&list_tmp);
+    excluded_items = 0;
+    size = (sizeof(struct flb_kv *) * (all_items));
+    arr = flb_calloc(1, size);
     if (!arr) {
         flb_errno();
         flb_kv_release(&list_tmp);
@@ -807,10 +838,32 @@ static flb_sds_t flb_signv4_canonical_request(struct flb_http_client *c,
     i = 0;
     mk_list_foreach(head, &list_tmp) {
         kv = mk_list_entry(head, struct flb_kv, _head);
+
+        /* Skip excluded headers */
+        if (excluded_headers) {
+            skip_header = FLB_FALSE;
+            mk_list_foreach(head_2, excluded_headers) {
+                sle = mk_list_entry(head_2, struct flb_slist_entry, _head);
+                if (flb_sds_casecmp(kv->key, sle->str, flb_sds_len(sle->str)) == 0) {
+
+                    /* Skip header */
+                    excluded_items++;
+                    skip_header = FLB_TRUE;
+                    break;
+                }
+            }
+            if (skip_header) {
+                continue;
+            }
+        }
+
         arr[i] = kv;
         i++;
     }
 
+    /* Count items */
+    items = all_items - excluded_items;
+
     /* Sort the headers from the temporal array */
     qsort(arr, items, sizeof(struct flb_kv *), kv_key_cmp);
 
@@ -913,10 +966,10 @@ static flb_sds_t flb_signv4_string_to_sign(struct flb_http_client *c,
                                            char *region)
 {
     int i;
+    int result;
     flb_sds_t tmp;
     flb_sds_t sign;
     unsigned char sha256_buf[64] = {0};
-    mbedtls_sha256_context sha256_ctx;
 
     sign = flb_sds_create_size(256);
     if (!sign) {
@@ -952,10 +1005,15 @@ static flb_sds_t flb_signv4_string_to_sign(struct flb_http_client *c,
     }
 
     /* Hash of Canonical Request */
-    mbedtls_sha256_init(&sha256_ctx);
-    mbedtls_sha256_starts(&sha256_ctx, 0);
-    mbedtls_sha256_update(&sha256_ctx, (unsigned char *) cr, flb_sds_len(cr));
-    mbedtls_sha256_finish(&sha256_ctx, sha256_buf);
+    result = flb_hash_simple(FLB_HASH_SHA256,
+                             (unsigned char *) cr, flb_sds_len(cr),
+                             sha256_buf, sizeof(sha256_buf));
+
+    if (result != FLB_CRYPTO_SUCCESS) {
+        flb_error("[signv4] error hashing canonical request");
+        flb_sds_destroy(sign);
+        return NULL;
+    }
 
     for (i = 0; i < 32; i++) {
         tmp = flb_sds_printf(&sign, "%02x", (unsigned char) sha256_buf[i]);
@@ -1086,6 +1144,7 @@ flb_sds_t flb_signv4_do(struct flb_http_client *c, int normalize_uri,
                         time_t t_now,
                         char *region, char *service,
                         int s3_mode,
+                        struct mk_list *unsigned_headers,
                         struct flb_aws_provider *provider)
 {
     char amzdate[32];
@@ -1105,7 +1164,7 @@ flb_sds_t flb_signv4_do(struct flb_http_client *c, int normalize_uri,
         return NULL;
     }
 
-    gmt = flb_malloc(sizeof(struct tm));
+    gmt = flb_calloc(1, sizeof(struct tm));
     if (!gmt) {
         flb_errno();
         flb_aws_credentials_destroy(creds);
@@ -1134,6 +1193,7 @@ flb_sds_t flb_signv4_do(struct flb_http_client *c, int normalize_uri,
     cr = flb_signv4_canonical_request(c, normalize_uri,
                                       amz_date_header, amzdate,
                                       creds->session_token, s3_mode,
+                                      unsigned_headers,
                                       &signed_headers);
     if (!cr) {
         flb_error("[signv4] failed canonical request");
