@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -59,7 +58,7 @@ struct flb_ml_parser *flb_ml_parser_java(struct flb_config *config, char *key)
 
     ret = rule(mlp,
                "start_state, java_start_exception",
-               "/(?:Exception|Error|Throwable|V8 errors stack trace)[:\\r\\n]/",
+               "/(.)(?:Exception|Error|Throwable|V8 errors stack trace)[:\\r\\n]/",
                "java_after_exception", NULL);
     if (ret != 0) {
         rule_error(mlp);
@@ -135,7 +134,7 @@ struct flb_ml_parser *flb_ml_parser_java(struct flb_config *config, char *key)
     /* Map the rules (mandatory for regex rules) */
     ret = flb_ml_parser_init(mlp);
     if (ret != 0) {
-        flb_error("[multiline: python] error on mapping rules");
+        flb_error("[multiline: java] error on mapping rules");
         flb_ml_parser_destroy(mlp);
         return NULL;
     }
